@@ -1,13 +1,8 @@
-import typing
-from typing import Any, Callable, List, Tuple, Union
-
 from .abc_interpreter import Interpreter
 from ..data_processor.readers import preprocess_inputs, preprocess_save_path
-from ..data_processor.visualizer import visualize_grayscale
+from ..data_processor.visualizer import explanation_to_vis, show_vis_explanation, save_image
 
 import numpy as np
-import paddle.fluid as fluid
-import os, sys
 import paddle
 
 
@@ -33,10 +28,13 @@ class GradShapCVInterpreter(Interpreter):
         """
         Interpreter.__init__(self)
         self.paddle_model = paddle_model
-        self.use_cuda = use_cuda
         self.model_input_shape = model_input_shape
         self.paddle_prepared = False
 
+        self.use_cuda = use_cuda
+        if not paddle.is_compiled_with_cuda():
+            self.use_cuda = False
+
     def interpret(self,
                   inputs,
                   labels=None,
@@ -74,7 +72,7 @@ class GradShapCVInterpreter(Interpreter):
             repeated_data = np.repeat(data, (n_samples, ) * len(data), axis=0)
             return repeated_data + noise
 
-        _, data = preprocess_inputs(inputs, self.model_input_shape)
+        imgs, data = preprocess_inputs(inputs, self.model_input_shape)
 
         bsz = len(data)
         save_path = preprocess_save_path(save_path, bsz)
@@ -119,19 +117,19 @@ class GradShapCVInterpreter(Interpreter):
                 keepdims=True) for i in range(bsz)
         ])
 
+        # visualization and save image.
         for i in range(bsz):
-            visualize_grayscale(
-                interpretations[i], visual=visual, save_path=save_path[i])
+            vis_explanation = explanation_to_vis(imgs[i], np.abs(interpretations[i]).sum(0), style='overlay_grayscale')
+            if visual:
+                show_vis_explanation(vis_explanation)
+            if save_path[i] is not None:
+                save_image(save_path[i], vis_explanation)
 
         return interpretations
 
     def _paddle_prepare(self, predict_fn=None):
         if predict_fn is None:
-            if self.use_cuda:
-                paddle.set_device('gpu:0')
-            else:
-                paddle.set_device('cpu')
-
+            paddle.set_device('gpu:0' if self.use_cuda else 'cpu')
             self.paddle_model.train()
 
             for n, v in self.paddle_model.named_sublayers():
