@@ -1,12 +1,10 @@
-import typing
-from typing import Any, Callable, List, Tuple, Union
+
 import numpy as np
-import os, sys
 import paddle
 
 from .abc_interpreter import Interpreter
 from ..data_processor.readers import preprocess_inputs, preprocess_save_path
-from ..data_processor.visualizer import visualize_overlay
+from ..data_processor.visualizer import explanation_to_vis, show_vis_explanation, save_image
 
 
 class IntGradCVInterpreter(Interpreter):
@@ -32,10 +30,13 @@ class IntGradCVInterpreter(Interpreter):
         """
         Interpreter.__init__(self)
         self.paddle_model = paddle_model
-        self.use_cuda = use_cuda
         self.model_input_shape = model_input_shape
         self.paddle_prepared = False
 
+        self.use_cuda = use_cuda
+        if not paddle.is_compiled_with_cuda():
+            self.use_cuda = False
+
     def interpret(self,
                   inputs,
                   labels=None,
@@ -106,18 +107,19 @@ class IntGradCVInterpreter(Interpreter):
             gradients_list.append(ig_gradients)
         avg_gradients = np.average(np.array(gradients_list), axis=0)
 
+        # visualization and save image.
         for i in range(len(imgs)):
-            visualize_overlay(avg_gradients[i], imgs[i], visual, save_path[i])
+            vis_explanation = explanation_to_vis(imgs[i], np.abs(avg_gradients[i]).sum(0), style='overlay_grayscale')
+            if visual:
+                show_vis_explanation(vis_explanation)
+            if save_path[i] is not None:
+                save_image(save_path[i], vis_explanation)
 
         return avg_gradients
 
     def _paddle_prepare(self, predict_fn=None):
         if predict_fn is None:
-            if self.use_cuda:
-                paddle.set_device('gpu:0')
-            else:
-                paddle.set_device('cpu')
-
+            paddle.set_device('gpu:0' if self.use_cuda else 'cpu')
             self.paddle_model.train()
 
             for n, v in self.paddle_model.named_sublayers():
