@@ -1,12 +1,9 @@
-import os
-import typing
-from typing import Any, Callable, List, Tuple, Union
+
 import numpy as np
 import paddle
 
 from ..data_processor.readers import preprocess_image, read_image, restore_image
-from ..data_processor.visualizer import show_important_parts, visualize_image, save_image
-from ..common.paddle_utils import init_checkpoint, to_lodtensor
+from ..data_processor.visualizer import sp_weights_to_image_explanation, overlay_threshold, save_image, show_vis_explanation
 
 from ._lime_base import LimeBase
 from .abc_interpreter import Interpreter
@@ -21,9 +18,10 @@ class LIMECVInterpreter(Interpreter):
     """
 
     def __init__(self,
-                 paddle_model: Callable,
+                 paddle_model,
                  model_input_shape=[3, 224, 224],
-                 use_cuda=True) -> None:
+                 use_cuda=True,
+                 random_seed=None) -> None:
         """
         Initialize the LIMECVInterpreter.
 
@@ -36,13 +34,16 @@ class LIMECVInterpreter(Interpreter):
         Interpreter.__init__(self)
         self.paddle_model = paddle_model
         self.model_input_shape = model_input_shape
-        self.use_cuda = use_cuda
         self.paddle_prepared = False
 
-        # use the default LIME setting
-        self.lime_base = LimeBase()
+        self.use_cuda = use_cuda
+        if not paddle.is_compiled_with_cuda():
+            self.use_cuda = False
 
-        self.lime_intermediate_results = {}
+        # use the default LIME setting
+        self.lime_base = LimeBase(random_state=random_seed)
+
+        self.lime_results = {}
 
     def interpret(self,
                   data,
@@ -77,9 +78,6 @@ class LIMECVInterpreter(Interpreter):
             else:
                 data_instance = restore_image(data.copy())
 
-        self.input_type = type(data_instance)
-        self.data_type = np.array(data_instance).dtype
-
         if not self.paddle_prepared:
             self._paddle_prepare()
         # only one example here
@@ -99,43 +97,34 @@ class LIMECVInterpreter(Interpreter):
             num_samples=num_samples,
             batch_size=batch_size)
 
-        interpretation = show_important_parts(
-            data_instance[0],
-            lime_weights,
-            interpret_class[0],
-            self.lime_base.segments,
-            visual=visual,
-            save_path=save_path)
+        # visualization and save image.
+        explanation_mask = sp_weights_to_image_explanation(
+            data_instance[0], lime_weights, interpret_class[0], self.lime_base.segments
+        )
+        explanation_vis = overlay_threshold(data_instance[0], explanation_mask)
+        if visual:
+            show_vis_explanation(explanation_vis)
+        if save_path is not None:
+            save_image(save_path, explanation_vis)
 
-        self.lime_intermediate_results['probability'] = probability
-        self.lime_intermediate_results['input'] = data_instance[0]
-        self.lime_intermediate_results[
-            'segmentation'] = self.lime_base.segments
-        self.lime_intermediate_results['r2_scores'] = r2_scores
+        self.lime_results['probability'] = probability
+        self.lime_results['input'] = data_instance[0]
+        self.lime_results['segmentation'] = self.lime_base.segments
+        self.lime_results['r2_scores'] = r2_scores
+        self.lime_results['lime_weights'] = lime_weights
 
         return lime_weights
 
     def _paddle_prepare(self, predict_fn=None):
         if predict_fn is None:
-            if self.use_cuda:
-                paddle.set_device('gpu:0')
-            else:
-                paddle.set_device('cpu')
-
-            self.paddle_model.train()
-
-            for n, v in self.paddle_model.named_sublayers():
-                if "batchnorm" in v.__class__.__name__.lower():
-                    v._use_global_stats = True
-                if "dropout" in v.__class__.__name__.lower():
-                    v.p = 0
+            paddle.set_device('gpu:0' if self.use_cuda else 'cpu')
+            self.paddle_model.eval()
 
             def predict_fn(data_instance):
                 data = preprocess_image(
                     data_instance
                 )  # transpose to [N, 3, H, W], scaled to [0.0, 1.0]
                 data = paddle.to_tensor(data)
-                data.stop_gradient = False
                 out = self.paddle_model(data)
                 probs = paddle.nn.functional.softmax(out, axis=1)
                 return probs.numpy()
@@ -152,7 +141,7 @@ class LIMENLPInterpreter(Interpreter):
     https://arxiv.org/abs/1602.04938
     """
 
-    def __init__(self, paddle_model, use_cuda=True) -> None:
+    def __init__(self, paddle_model, use_cuda=True, random_seed=None) -> None:
         """
         Initialize the LIMENLPInterpreter.
 
@@ -166,10 +155,13 @@ class LIMENLPInterpreter(Interpreter):
         Interpreter.__init__(self)
         self.paddle_model = paddle_model
         self.use_cuda = use_cuda
+        if not paddle.is_compiled_with_cuda():
+            self.use_cuda = False
+
         self.paddle_prepared = False
 
         # use the default LIME setting
-        self.lime_base = LimeBase()
+        self.lime_base = LimeBase(random_state=random_seed)
 
         self.lime_intermediate_results = {}
 
@@ -242,18 +234,8 @@ class LIMENLPInterpreter(Interpreter):
 
     def _paddle_prepare(self, predict_fn=None):
         if predict_fn is None:
-            if self.use_cuda:
-                paddle.set_device('gpu:0')
-            else:
-                paddle.set_device('cpu')
-
-            self.paddle_model.train()
-
-            for n, v in self.paddle_model.named_sublayers():
-                if "batchnorm" in v.__class__.__name__.lower():
-                    v._use_global_stats = True
-                if "dropout" in v.__class__.__name__.lower():
-                    v.p = 0
+            paddle.set_device('gpu:0' if self.use_cuda else 'cpu')
+            self.paddle_model.eval()
 
             def predict_fn(*params):
                 params = tuple(paddle.to_tensor(inp) for inp in params)
